#!/bin/bash
#
#  pasmo		... assembler
#  python3 ips_util	... to make ips-file
#
src_name="StarSoldier"
src_rom="${src_name}.rom"
dst_rom="${src_name}-ss-v0.0.rom"
dst_ips="${src_name}-ss-v0.0.ips"
tmp_rom="${src_name}-ss-v0.0.tmp.rom"
af="${src_name}-ss.asm"
sb="${src_name}-ss.symbol"


if [ "${1}" = "-f" ] ; then
  rm -f "${dst_rom}"
  rm -f "${dst_ips}"
fi



if ! [ -d "obj" ] ; then
  mkdir -p obj
fi


function isnew {
  if [ "${1}" -nt "${dst_rom}" ] ; then return 1 ; fi
  for i in `sed -nre "s/^[ \t]*inc(lude|bin)[ \t]*\"(.+)\".*/\\2/p" "${1}"` ; do
    isnew "${i}"
    if [ $? = "1" ] ; then return 1 ; fi
  done
  return 0
}
if [ -f "${dst_rom}" ] ; then
  isnew "${af}"
  if [ "$?" != "1" ] ; then
    echo "nothing to do."
    exit 0
  fi
fi


echo "##"
echo "## assemble ${af}"
echo "##"
pasmo --alocal "${af}" "${tmp_rom}" "obj/${sb}"
if [ "$?" != "0" ] ; then exit 1; fi


echo "##"
echo "## bytes replace from obj/${sb}"
echo "##"
a=(`cat "obj/${sb}" | sed -nre "s/val_[^_]{2}_in_rom_([^ ]{4,5}) EQU 000([^H]{2})H/\\1 \\2/p"`)
echo "${a[@]}"
while [ "${#a[@]}" -ge "2" ] ; do
  s=$((16#${a[0]}))
  #echo "${sb}" "${a[0]}" "${a[1]}" "${s}"
  printf "\\x${a[1]}" | dd of="${tmp_rom}" bs=1 count=1 seek="${s}" conv=notrunc 2>/dev/null
  unset a[0]
  unset a[1]
  a=("${a[@]}")
done


echo "##"
echo "## words replace from obj/${sb}"
echo "##"
a=(`cat "obj/${sb}" | sed -nre "s/val_[^_]{4}_in_rom_([^ ]{4,5}) EQU 0([^H]{2})([^H]{2})H/\\1 \\2 \\3/p"`)
echo "${a[@]}"
while [ "${#a[@]}" -ge "3" ] ; do
  s=$((16#${a[0]}))
  #echo "${sb}" "${a[0]}" "${a[1]}" "${a[2]}" "${s}"
  printf "\\x${a[2]}\\x${a[1]}" | dd of="${tmp_rom}" bs=1 count=2 seek="${s}" conv=notrunc 2>/dev/null
  unset a[0]
  unset a[1]
  unset a[2]
  a=("${a[@]}")
done



echo "##"
echo "## make rom & ips"
echo "##"
cmp -s "${tmp_rom}" "${dst_rom}"
if  [ "$?" != "0" ] ; then
  python3 -m ips_util create -o "${dst_ips}" "${src_rom}" "${tmp_rom}"
  mv -v "${tmp_rom}" "${dst_rom}"
else
  echo "build done, but nothing to change."
fi
rm -f "${tmp_rom}"
